@@ -94,7 +94,7 @@ func (f *File) IsDir() bool {
 func (f *File) Mode() (mode os.FileMode) {
 	f.mu.RLock()
 	defer f.mu.RUnlock()
-	mode = f.d.vfs.Opt.FilePerms
+	mode = os.FileMode(f.d.vfs.Opt.FilePerms)
 	if f.appendMode {
 		mode |= os.ModeAppend
 	}
