@@ -10,7 +10,11 @@ import (
 	"time"
 
 	_ "github.com/rclone/rclone/backend/local" // import the local backend
+	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/config"
 	"github.com/rclone/rclone/fstest"
+	"github.com/rclone/rclone/lib/diskusage"
+	"github.com/rclone/rclone/vfs/vfscache/writeback"
 	"github.com/rclone/rclone/vfs/vfscommon"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -101,7 +105,7 @@ func newTestCacheOpt(t *testing.T, opt vfscommon.Options) (r *fstest.Run, c *Cac
 }
 
 func newTestCache(t *testing.T) (r *fstest.Run, c *Cache) {
-	opt := vfscommon.DefaultOpt
+	opt := vfscommon.Opt
 
 	// Disable the cache cleaner as it interferes with these tests
 	opt.CachePollInterval = 0
@@ -354,7 +358,8 @@ func TestCachePurgeOverQuota(t *testing.T) {
 	}, itemAsString(c))
 
 	// Check nothing removed
-	c.purgeOverQuota(1)
+	c.opt.CacheMaxSize = 1
+	c.purgeOverQuota()
 
 	// Close the files
 	require.NoError(t, potato.Close(nil))
@@ -373,7 +378,8 @@ func TestCachePurgeOverQuota(t *testing.T) {
 	potato2.info.ATime = t1
 
 	// Check only potato removed to get below quota
-	c.purgeOverQuota(10)
+	c.opt.CacheMaxSize = 10
+	c.purgeOverQuota()
 	assert.Equal(t, int64(6), c.used)
 
 	assert.Equal(t, []string{
@@ -399,7 +405,8 @@ func TestCachePurgeOverQuota(t *testing.T) {
 	potato.info.ATime = t2
 
 	// Check only potato2 removed to get below quota
-	c.purgeOverQuota(10)
+	c.opt.CacheMaxSize = 10
+	c.purgeOverQuota()
 	assert.Equal(t, int64(5), c.used)
 	c.purgeEmptyDirs("", true)
 
@@ -408,7 +415,8 @@ func TestCachePurgeOverQuota(t *testing.T) {
 	}, itemAsString(c))
 
 	// Now purge everything
-	c.purgeOverQuota(1)
+	c.opt.CacheMaxSize = 1
+	c.purgeOverQuota()
 	assert.Equal(t, int64(0), c.used)
 	c.purgeEmptyDirs("", true)
 
@@ -420,6 +428,26 @@ func TestCachePurgeOverQuota(t *testing.T) {
 	assert.Equal(t, []string(nil), itemAsString(c))
 }
 
+func TestCachePurgeMinFreeSpace(t *testing.T) {
+	du, err := diskusage.New(config.GetCacheDir())
+	if err == diskusage.ErrUnsupported {
+		t.Skip(err)
+	}
+	// We've tested the quota mechanism already, so just test the
+	// min free space quota is working.
+	_, c := newTestCache(t)
+
+	// First set free space quota very small and check it is OK
+	c.opt.CacheMinFreeSpace = 1
+	assert.True(t, c.minFreeSpaceQuotaOK())
+	assert.True(t, c.quotasOK())
+
+	// Now set it a bit larger than the current disk available and check it is BAD
+	c.opt.CacheMinFreeSpace = fs.SizeSuffix(du.Available) + fs.Gibi
+	assert.False(t, c.minFreeSpaceQuotaOK())
+	assert.False(t, c.quotasOK())
+}
+
 // test reset clean files
 func TestCachePurgeClean(t *testing.T) {
 	r, c := newItemTestCache(t)
@@ -453,7 +481,8 @@ func TestCachePurgeClean(t *testing.T) {
 	require.NoError(t, potato3.Truncate(6))
 
 	c.updateUsed()
-	c.purgeClean(1)
+	c.opt.CacheMaxSize = 1
+	c.purgeClean()
 	assert.Equal(t, []string{
 		`name="existing" opens=2 size=100 space=0`,
 		`name="sub/dir/potato2" opens=1 size=5 space=5`,
@@ -462,7 +491,8 @@ func TestCachePurgeClean(t *testing.T) {
 	assert.Equal(t, int64(11), c.used)
 
 	require.NoError(t, potato2.Close(nil))
-	c.purgeClean(1)
+	c.opt.CacheMaxSize = 1
+	c.purgeClean()
 	assert.Equal(t, []string{
 		`name="existing" opens=2 size=100 space=0`,
 		`name="sub/dir/potato3" opens=1 size=6 space=6`,
@@ -476,7 +506,8 @@ func TestCachePurgeClean(t *testing.T) {
 	// Remove all files now.  The are all not in use.
 	// purgeClean does not remove empty cache files. purgeOverQuota does.
 	// So we use purgeOverQuota here for the cleanup.
-	c.purgeOverQuota(1)
+	c.opt.CacheMaxSize = 1
+	c.purgeOverQuota()
 
 	c.purgeEmptyDirs("", true)
 
@@ -597,12 +628,12 @@ func TestCacheRename(t *testing.T) {
 }
 
 func TestCacheCleaner(t *testing.T) {
-	opt := vfscommon.DefaultOpt
-	opt.CachePollInterval = 10 * time.Millisecond
-	opt.CacheMaxAge = 20 * time.Millisecond
+	opt := vfscommon.Opt
+	opt.CachePollInterval = fs.Duration(10 * time.Millisecond)
+	opt.CacheMaxAge = fs.Duration(20 * time.Millisecond)
 	_, c := newTestCacheOpt(t, opt)
 
-	time.Sleep(2 * opt.CachePollInterval)
+	time.Sleep(time.Duration(2 * opt.CachePollInterval))
 
 	potato := c.Item("potato")
 	potato2, found := c.get("potato")
@@ -610,7 +641,7 @@ func TestCacheCleaner(t *testing.T) {
 	assert.True(t, found)
 
 	for i := 0; i < 100; i++ {
-		time.Sleep(10 * opt.CachePollInterval)
+		time.Sleep(time.Duration(10 * opt.CachePollInterval))
 		potato2, found = c.get("potato")
 		if !found {
 			break
@@ -697,3 +728,26 @@ func TestCacheStats(t *testing.T) {
 	assert.Equal(t, 0, out["uploadsInProgress"])
 	assert.Equal(t, 0, out["uploadsQueued"])
 }
+
+func TestCacheQueue(t *testing.T) {
+	_, c := newTestCache(t)
+
+	out := c.Queue()
+
+	// We've checked the contents of queue in the writeback tests
+	// Just check it is present here
+	queue, found := out["queue"]
+	require.True(t, found)
+	_, ok := queue.([]writeback.QueueInfo)
+	require.True(t, ok)
+}
+
+func TestCacheQueueSetExpiry(t *testing.T) {
+	_, c := newTestCache(t)
+
+	// Check this returns the correct error when called so we know
+	// it is plumbed in correctly. The actual tests are done in
+	// writeback.
+	err := c.QueueSetExpiry(123123, time.Now())
+	assert.Equal(t, writeback.ErrorIDNotFound, err)
+}
